
#ifndef CODEGEN_H_INCLUDED
#define CODEGEN_H_INCLUDED

#include "minako.h"

/* *************************************************************** Strukturen */

/* anonyme Strukturen verhindern den Zugriff auf interne Daten von außerhalb */
typedef struct sCodegen codegen;

/* ***************************************************************** Methoden */

/**@brief Erstellt eine neue Codegenerierungskomponenteninstanz (puh).
 * @param className   Name der zu generierenden Java-Klasse
 * 
 * @return Zeiger auf die Codegenerierungskomponenteninstanz (puh²)
 */
extern codegen* newCodegen(const char* className);

/**@brief Schreibt alle generierten Daten in yyout und löscht das Objekt.
 */
extern void deleteCodegen(codegen* gen);

/**@brief Hängt einen Befehl an die Ausgabe an.
 * @param gen        Codegenerierung
 * @param cmd        Befehl
 * @param stackDiff  Veränderung der Stackgröße
 */
extern void codegenAppendCommand(codegen* gen, const char* cmd, int stackDiff);

/**@brief Fügt einen Befehl in die Ausgabe ein.
 * @param gen        Codegenerierung
 * @param address    zurückgegebener Zeiger aus einem früheren Aufruf von
 *                   codegenCurrentAddress
 * @param cmd        Befehl
 * @param stackDiff  Veränderung der Stackgröße
 */
extern void codegenInsertCommand(codegen* gen, const void* address,
                                 const char* cmd, int stackDiff);

/**@brief Fügt ein neues Sprunglabel in die Ausgabe ein.
 * @param gen        Codegenerierung
 * @param label      Nummer eines Labels
 */
extern void codegenAppendLabel(codegen* gen, unsigned int label);

/**@brief Fügt eine Variablendeklaration in die Ausgabe ein.
 * @param gen        Codegenerierung
 * @param name       Name der Variablen
 * @param var        Minako Variable
 * @note Der Datentyp gehorcht den Deklarationen aus der minako.h.
 */
extern void codegenAddVariable(codegen* gen, const char* name,
                               minakoVariable* var);

/**@brief Signalisiert der Codegenerierungskomponente, dass eine Variable ihren
 * Gültigkeitsbereich verlassen hat.
 * @param gen        Codegenerierung
 * @param var        Minako Variable
 */
extern void codegenRemoveVariable(codegen* gen, minakoVariable* var);

/**@brief Generiert die Zuweisung einer Variablen.
 * @param gen        Codegenerierung
 * @param name       Name der Variablen
 * @param var        Minako Variable
 */
extern void codegenAssignVariable(codegen* gen, const char* name,
                                  minakoVariable* var);

/**@brief Generiert eine neue, unverbrauchte Labelnummer.
 * @param gen        Codegenerierung
 * 
 * @return frische Labelnummer
 */
extern unsigned int codegenNextLabel(codegen* gen);

/**@brief Gibt die aktuelle Adresse in der Codegenerierung zurück.
 */
extern const void* codegenCurrentAddress(const codegen* gen);

/**@brief Generiert Code für den Eintritt in eine neue C1-Funktion.
 */
extern void codegenEnterFunction(codegen* gen, const char* name,
                                 const minakoFunction* func);

/**@brief Generiert Code für den Austritt aus der aktuellen C1-Funktion.
 */
extern void codegenLeaveFunction(codegen* gen);

#endif
