
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "stringbuffer.h"

/* *************************************************************** Strukturen */

typedef struct sElement
{
	char* datum;
	struct sElement* succ;
} element;

struct sStringBuffer
{
	element *top, *bottom;
};

/* ******************************************************* private Funktionen */

static void terminate(const char* description)
{
	fprintf(stderr, "Error: %s\n", description);
	exit(-1);
}

/* *************************************************** exportierte Funktionen */

stringBuffer* newStringBuffer()
{
	stringBuffer* res = (stringBuffer*) calloc(1, sizeof(stringBuffer));
	
	if (!res)
		terminate("Memory limit exceeded");
	
	return res;
}

void deleteStringBuffer(stringBuffer* buffer)
{
	if (buffer)
	{
		while (buffer->top)
		{
			buffer->bottom = buffer->top->succ;
			
			free(buffer->top->datum);
			free(buffer->top);
			
			buffer->top = buffer->bottom;
		}
		
		free(buffer);
	}
	else
		terminate("NULL pointer exception (deleteStringBuffer)");
}

void stringBufferAppend(stringBuffer* buffer, const char* text)
{
	if (buffer && text)
	{
		element* newElement = (element*) malloc(sizeof(element));
		
		if (!newElement)
			terminate("Memory limit exceeded");
		
		newElement->datum = strdup(text);
		newElement->succ = 0;
		
		if (buffer->top)
		{
			buffer->bottom->succ = newElement;
			buffer->bottom = newElement;
		}
		else
			buffer->top = buffer->bottom = newElement;
	}
	else
		terminate("NULL pointer exception (stringBufferAppend)");
}

void stringBufferInsert(stringBuffer* buffer, const void* address,
                        const char* text)
{
	if (buffer && text)
	{
		element *curr = (element*) address,
		        *next = (element*) malloc(sizeof(element));
		
		if (!next)
			terminate("Memory limit exceeded");
		
		next->datum = strdup(text);
		
		/* behandle den Fall, dass die Adresse vom aktuell letzten Element stammt */
		if (curr == buffer->bottom)
			buffer->bottom = next;
		
		if (curr)
		{
			/* Standardfall */
			next->succ = curr->succ;
			curr->succ = next;
		}
		else
		{
			/* die Adresse ist vor dem ersten Element */
			next->succ = buffer->top;
			buffer->top = next;
		}
	}
	else
		terminate("NULL pointer exception (stringBufferInsert)");
}

const void* stringBufferGetCurrentAddress(const stringBuffer* buffer)
{
	if (!buffer)
		terminate("NULL pointer exception (stringBufferGetCurrentAddress)");
	
	return buffer->bottom;
}

void stringBufferPrint(const stringBuffer* buffer, FILE* sOut)
{
	if (buffer && sOut)
	{
		if (buffer->top)
		{
			element* curr = buffer->top;
			
			do
				fprintf(sOut, "%s\n", curr->datum);
			while ((curr = curr->succ));
		}
		else
			fprintf(sOut, "<empty buffer>\n");
	}
	else
		terminate("NULL pointer exception (stringBufferPrint)");
}

void stringBufferConcatenate(stringBuffer* buf1, stringBuffer* buf2)
{
	if (buf1 && buf2)
	{
		if (buf2->top)
		{
			if (buf1->top)
			{
				buf1->bottom->succ = buf2->top;
				buf1->bottom = buf2->bottom;
			}
			else
			{
				buf1->top = buf2->top;
				buf1->bottom = buf2->bottom;
			}
			
			buf2->top = buf2->bottom = 0;
		}
		
		deleteStringBuffer(buf2);
	}
	else
		terminate("NULL pointer exception (stringBufferConcatenate)");
}

